#include "neslib.h"
#include "nam_pumpkin.h"

const unsigned char palette[16]={ 0x0f,0x00,0x10,0x30,0x0f,0x07,0x17,0x28,0x0f,0x09,0x1a,0x2a,0x0f,0x0f,0x0f,0x0f };

const unsigned char sprite_layer[]={
	  0,  0,0x6f,1,
	  8, 24,0xcf,0,
	 16, 24,0xd0,0,
	 24, 24,0xd1,0,
	 32, 24,0xd2,0,
	  8,  8,0xd3,1,
	128
};


void main(void)
{
	pal_bg(palette);
	pal_spr(palette);

	vram_adr(NAMETABLE_A);
	vram_unrle(nam_pumpkin);

	oam_meta_spr(13*8, 21*8-1, 0, sprite_layer);
	oam_spr(17*8, 15*8-1, 0xce, 2, 4);

	ppu_on_all();



	while(1);
}